' This programm allows picking up data from a figure
' and saving the numerical values to a file.


'  Last modified: Karl-Heinz Schmidt, 14. Febr. 2010


'    Copyright 2010: Dr. Karl-Heinz Schmidt, Rheinstrae 4, 64390 Erzhausen, Germany
'
'    This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.


' Short description:
' Display the figure from which the data are to be taken on the screen.
' Start Pickup.exe.
' Chose maximum transparency for the graphic window of Pickup and pull it on
' top of the figure to be scanned. *)
' Chose scales (linear or logarithmic.
' Enter the coordinates of the lower-left and upper-right point.
' Click on the lower-left and the upper-right point with the left mouse botton.
' Click on the different points to be picked up.
' Save the information in the edit window to a file.

' *) This feature is not provided by Windows XP, but it can be added
' by e.g. Glass2k.exe, which is available for free on the internet.


  nomainwin

on error goto [errorhandler]

' Default values
xscale$ = "lin"
yscale$ = "lin"
format$ = "satan"

ipoint = 0

WindowWidth = 160
WindowHeight = 360
UpperLeftX = 10
UpperLeftY = 10

statictext #controlwindow, "Pickup data from screen",10,10,150,15
statictext #controlwindow, "(C) K.-H. Schmidt, 2010",10,25,150,15

groupbox #controlwindow, "x scale",5,40,140,55
radiobutton #controlwindow.xlin,"linear",[xlin],[nil],50,52,100,20
radiobutton #controlwindow.xlog,"logarithmic",[xlog],[nil],50,72,100,20

groupbox #controlwindow, "y scale",5,95,140,55
radiobutton #controlwindow.ylin,"linear",[ylin],[nil],50,107,100,20
radiobutton #controlwindow.ylog,"logarithmic",[ylog],[nil],50,127,100,20

groupbox #controlwindow, "lower left point",5,150,140,55
statictext #controlwindow "xmin",50,164,30,20
textbox #controlwindow.xmin,90,161,50,20
statictext #controlwindow "ymin",50,184,30,20
textbox #controlwindow.ymin,90,181,50,20

groupbox #controlwindow, "upper right point",5,205,140,55
statictext #controlwindow "xmax",50,219,30,20
textbox #controlwindow.xmax,90,216,50,20
statictext #controlwindow "ymax",50,239,30,20
textbox #controlwindow.ymax,90,236,50,20

groupbox #controlwindow, "output format",5,260,140,55
radiobutton #controlwindow.raw, "only numbers",[raw],[nil],50,272,100,20
radiobutton #controlwindow.satan, "SATAN",[satan],[nil],50,292,100,20

open "Control" for window as #controlwindow
print #controlwindow, "trapclose [quit]"
print #controlwindow.xlin, "set"
print #controlwindow.ylin, "set"
print #controlwindow.satan, "set"


WindowWidth = 500
WindowHeight = 400
UpperLeftX = 250
UpperLeftY = 10

open "Make this window transparent and put it on top of the figure." for graphics as #graphicwindow
print #graphicwindow, "trapclose [quit]"
' print #graphicwindow, "backcolor red"
print #graphicwindow,"when leftButtonDown [put]"
print #graphicwindow, "when mouseMove [cursor]"


WindowWidth = 300
WindowHeight = 300
UpperLeftX = 10
UpperLeftY = 400

open "Output" for text as #outputwindow
print #outputwindow, "!trapclose [quit]"

print #outputwindow, "Help:"
print #outputwindow, "1. Select lin/log scale."
print #outputwindow, "2. Enter coordinates"
print #outputwindow, "   xmin, ymin, xmax, ymax."
print #outputwindow, "3. Click on lower left point."

wait


' Subroutines

function testinput()
  print #controlwindow.xmin,"!contents? xmin$"
  print #controlwindow.xmax,"!contents? xmax$"
  print #controlwindow.ymin,"!contents? ymin$"
  print #controlwindow.ymax,"!contents? ymax$"
  if xmin$ = "" or xmax$ = "" or ymin$ = "" or ymax$ = "" then
    testinput = 0
  else
    testinput = -1
  end if
end function

function getxmin()
  print #controlwindow.xmin,"!contents? xmin"
  getxmin = xmin
end function

function getxmax()
  print #controlwindow.xmax,"!contents? xmax"
  getxmax = xmax
end function

function getymin()
  print #controlwindow.ymin,"!contents? ymin"
  getymin = ymin
end function

function getymax()
  print #controlwindow.ymax,"!contents? ymax"
  getymax = ymax
end function

sub point x, y
  print #graphicwindow, "down ; size 1"
  print #graphicwindow, "place ";x-2;" ";y
  print #graphicwindow, "goto ";x+2;" ";y
  print #graphicwindow, "place ";x;" ";y-2
  print #graphicwindow, "goto ";x;" ";y+2
  print #graphicwindow, "flush"
end sub

sub fullcursor x,y,xold,yold
  print #graphicwindow, "home"
  print #graphicwindow, "posxy xcentral ycentral"
  xrange = 2 * xcentral
  yrange = 2 * ycentral
  print #graphicwindow, "down ; size 1"
  print #graphicwindow, "rule XOR"
  if xold >= 0 then
    print #graphicwindow, "place ";0;" ";yold
    print #graphicwindow, "goto ";xrange;" ";yold
  end if
  if yold >= 0 then
    print #graphicwindow, "place ";xold;" ";0
    print #graphicwindow, "goto ";xold;" ";yrange
  end if
  print #graphicwindow, "place ";0;" ";y
  print #graphicwindow, "goto ";xrange;" ";y
  print #graphicwindow, "place ";x;" ";0
  print #graphicwindow, "goto ";x;" ";yrange
  print #graphicwindow, "rule OVER"
end sub

' Branchings

[cursor]
xcursor = MouseX
ycursor = MouseY
call fullcursor xcursor,ycursor,xlast,ylast
xlast = xcursor
ylast = ycursor

wait

[put]
if testinput() = 0 then
  notice "Please enter xmin, xmax, ymin, ymax first"
  wait
end if
ipoint = ipoint + 1
' print ipoint
if ipoint = 1 then
  xmin = getxmin()
  ymin = getymin()
  xminpixel = MouseX
  yminpixel = MouseY
  call point xminpixel,yminpixel
  print #outputwindow,"4. Click on upper right point."
end if
if ipoint = 2 then
  xmax = getxmax()
  ymax = getymax()
  xmaxpixel = MouseX
  ymaxpixel = MouseY
  call point xmaxpixel,ymaxpixel
  print #outputwindow,"5. Click on data points."
  print #graphicwindow, "when mouseMove"
  call fullcursor -1,-1,xlast,ylast
end if
if ipoint = 3 then
  print #outputwindow,"!cls"
  if format$ = "satan" then
    print #outputwindow, "Pickupdata"
    print #outputwindow, "X:"
    print #outputwindow, "Y:"
    print #outputwindow, "H:  X    Y"
  end if
end if
if ipoint > 2 then
  xpixel = MouseX
  ypixel = MouseY
  call point xpixel,ypixel
  if xscale$ = "lin" then
    x = xmin + (xmax-xmin)/(xmaxpixel-xminpixel)*(xpixel-xminpixel)
  end if
  if xscale$ = "log" then
    x = exp(log(xmin) + log(xmax/xmin)/(xmaxpixel-xminpixel)*(xpixel-xminpixel))
  end if
  if yscale$ = "lin" then
    y = ymin + (ymax-ymin)/(ymaxpixel-yminpixel)*(ypixel-yminpixel)
  end if
  if yscale$ = "log" then
    y = exp(log(ymin) + log(ymax/ymin)/(ymaxpixel-yminpixel)*(ypixel-yminpixel))
  end if
  print #outputwindow, x;" ";y
end if


wait

[xlin]
xscale$ = "lin"
wait

[xlog]
xscale$ = "log"
wait

[ylin]
yscale$ = "lin"
wait

[ylog]
yscale$ = "log"
wait

[raw]
format$ = "raw"
wait

[satan]
format$ = "satan"
wait

[errorhandler]
notice "Error occured"
wait

[quit]
confirm "Really quit?"; answer$
if upper$(answer$) <> "YES" then wait
close #outputwindow
close #controlwindow
close #graphicwindow

end
